#include <imgui.h>
#include <imgui_stdlib.h>

#include "log.h"
#include "config.h"

static inline void write_config_and_update_structures(const Config& config) {
    try {
        write_config(config);
    } catch (const std::exception& e) {
        log("Failed to write config", e);
        return;
    }
}


static inline void settings_window(Config& config, float* config_write_timer, bool* show_settings_window) {
    if (!ImGui::Begin("Settings", show_settings_window)) {
        ImGui::End();
        return;
    }
    // TODO actually have process control
    ImGui::Text("Logcat command only takes effect when logcat is not running");
    if (ImGui::InputTextWithHint("Logcat command", "adb logcat -Dv 'threadtime UTC epoch usec'", &config.logcat_command)) {
        *config_write_timer = *config_write_timer > 0.0f ? *config_write_timer : 5.0f;
    }
    ImGui::End();
}

static inline void logs_window(bool* show_logs_window) {
    if (!ImGui::Begin("LogMeow Logs", show_logs_window)) {
        ImGui::End();
        return;
    }

    if (ImGui::Button("Clear")) {
        log_entries.clear();
    }
    ImGui::SameLine();
    if (ImGui::Button("Copy")) {
        ImGui::SetClipboardText(log_entries.c_str());
    }

    ImGui::Separator();
    // copied from imgui/imgui_demo.cpp: [SECTION] Example App: Debug Console / ShowExampleAppConsole()
    if (ImGui::BeginChild("ScrollingRegion", ImVec2(0, 0), false, ImGuiWindowFlags_HorizontalScrollbar)) {
        ImGui::TextUnformatted(log_entries.data(), &log_entries[log_entries.size()]);
        if (ImGui::GetScrollY() >= ImGui::GetScrollMaxY()) {
            ImGui::SetScrollHereY(1.0f);
        }
    }
    ImGui::EndChild();
    ImGui::End();
}

static inline void main_window(bool* show_settings_window, bool* show_logs_window, bool* run_event_loop) {
    if (!ImGui::Begin("LogMeow", run_event_loop)) {
        ImGui::End();
        return;
    }
    if (ImGui::Button("Settings")) {
        *show_settings_window = true;
    }
    ImGui::SameLine();
    if (ImGui::Button("Logs")) {
        *show_logs_window = true;
    }
    ImGui::End();
}

static inline void debug_window() {
    static bool show_demo_window = false;
    static size_t add_log_entry_presses = 1;
    static bool log_entry_every_second = false;
    static float log_entry_every_second_delta;

    if (show_demo_window) {
        ImGui::ShowDemoWindow(&show_demo_window);
    }
    if (!ImGui::Begin("LogMeow Debug")) {
        ImGui::End();
        return;
    }

    ImGui::Text("Application average %.3f ms/frame (%.1f FPS)", 1000.0f / ImGui::GetIO().Framerate, ImGui::GetIO().Framerate);
    ImGui::Checkbox("Show Dear ImGui Demo Window", &show_demo_window);

    ImGui::Separator();
    if (ImGui::Button("Add Log Entry")) {
        std::exception e;
        log(std::string("Debug log entry #") + std::to_string(add_log_entry_presses++) + " (activated via manual button press)", e);
    }
    ImGui::SameLine();
    // returns true when it's pressed
    if (ImGui::Checkbox("Add log entry every second", &log_entry_every_second)) {
        log_entry_every_second_delta = 0.0f;
    }
    if (log_entry_every_second) {
        log_entry_every_second_delta += ImGui::GetIO().DeltaTime;
        if (log_entry_every_second_delta >= 1.0f) {
            std::exception e;
            log_entry_every_second_delta = 0.0f;
            log(std::string("Debug log entry #") + std::to_string(add_log_entry_presses++) + " (activated by add log entry every second)", e);
        }
    }

    ImGui::End();
}

void event_loop(Config& config, float* config_write_timer, bool* run_event_loop) {
    static bool show_settings_window = false;
    static bool show_logs_window = false;

    debug_window();

    if (show_settings_window) {
        settings_window(config, config_write_timer, &show_settings_window);
    }
    if (*config_write_timer > 0.0f) {
        *config_write_timer -= ImGui::GetIO().DeltaTime;
        if (*config_write_timer <= 0.0f) {
            write_config_and_update_structures(config);
        }
    }

    if (show_logs_window) {
        logs_window(&show_logs_window);
    }

    main_window(&show_settings_window, &show_logs_window, run_event_loop);
}
